package university.persistence;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.LinkedList;
import java.util.List;

import university.model.Student;

/*
CREATE TABLE students
(
  code integer NOT NULL,
  firstname character varying(64) NOT NULL,
  lastname character varying(64) NOT NULL,
  birthdate date NOT NULL,
  CONSTRAINT pk_students PRIMARY KEY (code)
)
*/

public class StudentRepository {
	private DataSource dataSource;
	
	public StudentRepository() {
		dataSource = new DataSource();
	}
	
	public void persist(Student student) throws Exception {
		Connection connection = this.dataSource.getConnection();

		if (findByPrimaryKey(student.getCode())!=null) {
			System.err.println("the student already exists");
			return;
		}
				
		String insert = "insert into students(code, firstname, lastname, birthDate) values (?,?,?,?)";
		
		PreparedStatement statement = connection.prepareStatement(insert);
		statement.setInt(1, student.getCode());
		statement.setString(2, student.getFirstName());
		statement.setString(3, student.getLastName());
		statement.setDate(4, new java.sql.Date(student.getBirthDate().getTime()));
		statement.executeUpdate();
		
		// release resources
		statement.close();
		connection.close();
	}

	public void delete(Student student) throws Exception {
		Connection connection = this.dataSource.getConnection();

		String insert = "delete from students where code = ?";
		
		PreparedStatement statement = connection.prepareStatement(insert);
		statement.setInt(1, student.getCode());
		statement.executeUpdate();
			
		// release resources
		statement.close();
		connection.close();
	}

	public Student findByPrimaryKey(int code) throws Exception {
		
		Connection connection = this.dataSource.getConnection();
		String query = "select * from students where code=?";
		PreparedStatement statement = connection.prepareStatement(query);
		statement.setInt(1, code);
		ResultSet result = statement.executeQuery();
		Student student = null;
		if (result.next()) {
			long secs;
			java.util.Date birthDate; 
			student = new Student();
			student.setCode(result.getInt("code"));
			student.setFirstName(result.getString("firstname"));
			student.setLastName(result.getString("lastname"));
			secs = result.getDate("birthdate").getTime();
			birthDate = new java.util.Date(secs);
			student.setBirthDate(birthDate);
		}
		
		// release resources
		result.close();
		statement.close();
		connection.close();
		
		return student;
	}	
	
	public List<Student> findAll() throws Exception {
		List<Student> students = new LinkedList<Student>();
		Connection connection = this.dataSource.getConnection();
		String query = "select * from students";
		PreparedStatement statement = connection.prepareStatement(query);
		ResultSet result = statement.executeQuery();
		while(result.next()) {
			Student student = new Student();
			student.setCode(result.getInt("code"));
			student.setFirstName(result.getString("firstname"));
			student.setLastName(result.getString("lastname"));
			student.setBirthDate(new java.util.Date(result.getDate("birthdate").getTime()));
			students.add(student);
		}
		
		// release resources
		result.close();
		statement.close();
		connection.close();
		
		return students;
	}	

	public List<Student> findByBirthDate(java.util.Date birthDate) throws Exception {
		List<Student> students = new LinkedList<Student>();
		Connection connection = this.dataSource.getConnection();
		String query = "select * from students where birthdate=?";
		PreparedStatement statement = connection.prepareStatement(query);
		statement.setDate(1, new java.sql.Date(birthDate.getTime()));
		ResultSet result = statement.executeQuery();
		while(result.next()) {
			Student student = new Student();
			student.setCode(result.getInt("code"));
			student.setFirstName(result.getString("firstname"));
			student.setLastName(result.getString("lastname"));
			student.setBirthDate(new java.util.Date(result.getDate("birthdate").getTime()));
			students.add(student);
		}
			
		// release resources
		result.close();
		statement.close();
		connection.close();
		
		return students;
	}	
}
